@@ -918,6 +918,11 @@ assertFalse(desc.writable);
 assertFalse(desc.enumerable);
 assertFalse(desc.configurable);
 
+// Define non-array property, check that .length is unaffected.
+assertEquals(16, arr.length);
+Object.defineProperty(arr, '0x20', descElement);
+assertEquals(16, arr.length);
+
 // See issue 968: http://code.google.com/p/v8/issues/detail?id=968
 var o = { x : 42 };
 Object.defineProperty(o, "x", { writable: false });
@@ -1057,7 +1062,7 @@ assertEquals(999, o[999]);
 
 
 // Regression test: Bizzare behavior on non-strict arguments object.
-// TODO(mstarzinger): Tests disabled, see bug 2261
+// TODO(yangguo): Tests disabled, needs investigation!
 /*
 (function test(arg0) {
   // Here arguments[0] is a fast alias on arg0.
