@@ -830,82 +830,82 @@ class MinidumpReader(object):
 # };
 # static P p;
 INSTANCE_TYPES = {
-  64: "SYMBOL_TYPE",
-  68: "ASCII_SYMBOL_TYPE",
-  65: "CONS_SYMBOL_TYPE",
-  69: "CONS_ASCII_SYMBOL_TYPE",
-  66: "EXTERNAL_SYMBOL_TYPE",
-  74: "EXTERNAL_SYMBOL_WITH_ASCII_DATA_TYPE",
-  70: "EXTERNAL_ASCII_SYMBOL_TYPE",
-  82: "SHORT_EXTERNAL_SYMBOL_TYPE",
-  90: "SHORT_EXTERNAL_SYMBOL_WITH_ASCII_DATA_TYPE",
-  86: "SHORT_EXTERNAL_ASCII_SYMBOL_TYPE",
   0: "STRING_TYPE",
   4: "ASCII_STRING_TYPE",
   1: "CONS_STRING_TYPE",
   5: "CONS_ASCII_STRING_TYPE",
   3: "SLICED_STRING_TYPE",
   2: "EXTERNAL_STRING_TYPE",
-  10: "EXTERNAL_STRING_WITH_ASCII_DATA_TYPE",
   6: "EXTERNAL_ASCII_STRING_TYPE",
+  10: "EXTERNAL_STRING_WITH_ASCII_DATA_TYPE",
   18: "SHORT_EXTERNAL_STRING_TYPE",
-  26: "SHORT_EXTERNAL_STRING_WITH_ASCII_DATA_TYPE",
   22: "SHORT_EXTERNAL_ASCII_STRING_TYPE",
-  6: "PRIVATE_EXTERNAL_ASCII_STRING_TYPE",
-  128: "MAP_TYPE",
-  129: "CODE_TYPE",
-  130: "ODDBALL_TYPE",
-  131: "JS_GLOBAL_PROPERTY_CELL_TYPE",
-  132: "HEAP_NUMBER_TYPE",
-  133: "FOREIGN_TYPE",
-  134: "BYTE_ARRAY_TYPE",
-  135: "FREE_SPACE_TYPE",
-  136: "EXTERNAL_BYTE_ARRAY_TYPE",
-  137: "EXTERNAL_UNSIGNED_BYTE_ARRAY_TYPE",
-  138: "EXTERNAL_SHORT_ARRAY_TYPE",
-  139: "EXTERNAL_UNSIGNED_SHORT_ARRAY_TYPE",
-  140: "EXTERNAL_INT_ARRAY_TYPE",
-  141: "EXTERNAL_UNSIGNED_INT_ARRAY_TYPE",
-  142: "EXTERNAL_FLOAT_ARRAY_TYPE",
-  144: "EXTERNAL_PIXEL_ARRAY_TYPE",
-  146: "FILLER_TYPE",
-  147: "DECLARED_ACCESSOR_DESCRIPTOR_TYPE",
-  148: "DECLARED_ACCESSOR_INFO_TYPE",
-  149: "EXECUTABLE_ACCESSOR_INFO_TYPE",
-  150: "ACCESSOR_PAIR_TYPE",
-  151: "ACCESS_CHECK_INFO_TYPE",
-  152: "INTERCEPTOR_INFO_TYPE",
-  153: "CALL_HANDLER_INFO_TYPE",
-  154: "FUNCTION_TEMPLATE_INFO_TYPE",
-  155: "OBJECT_TEMPLATE_INFO_TYPE",
-  156: "SIGNATURE_INFO_TYPE",
-  157: "TYPE_SWITCH_INFO_TYPE",
-  158: "ALLOCATION_SITE_INFO_TYPE",
-  159: "SCRIPT_TYPE",
-  160: "CODE_CACHE_TYPE",
-  161: "POLYMORPHIC_CODE_CACHE_TYPE",
-  162: "TYPE_FEEDBACK_INFO_TYPE",
-  163: "ALIASED_ARGUMENTS_ENTRY_TYPE",
-  166: "FIXED_ARRAY_TYPE",
-  145: "FIXED_DOUBLE_ARRAY_TYPE",
-  167: "SHARED_FUNCTION_INFO_TYPE",
-  168: "JS_MESSAGE_OBJECT_TYPE",
-  171: "JS_VALUE_TYPE",
-  172: "JS_DATE_TYPE",
-  173: "JS_OBJECT_TYPE",
-  174: "JS_CONTEXT_EXTENSION_OBJECT_TYPE",
-  175: "JS_MODULE_TYPE",
-  176: "JS_GLOBAL_OBJECT_TYPE",
-  177: "JS_BUILTINS_OBJECT_TYPE",
-  178: "JS_GLOBAL_PROXY_TYPE",
-  179: "JS_ARRAY_TYPE",
-  170: "JS_PROXY_TYPE",
-  182: "JS_WEAK_MAP_TYPE",
-  183: "JS_REGEXP_TYPE",
-  184: "JS_FUNCTION_TYPE",
-  169: "JS_FUNCTION_PROXY_TYPE",
-  164: "DEBUG_INFO_TYPE",
-  165: "BREAK_POINT_INFO_TYPE",
+  26: "SHORT_EXTERNAL_STRING_WITH_ASCII_DATA_TYPE",
+  64: "INTERNALIZED_STRING_TYPE",
+  68: "ASCII_INTERNALIZED_STRING_TYPE",
+  65: "CONS_INTERNALIZED_STRING_TYPE",
+  69: "CONS_ASCII_INTERNALIZED_STRING_TYPE",
+  66: "EXTERNAL_INTERNALIZED_STRING_TYPE",
+  70: "EXTERNAL_ASCII_INTERNALIZED_STRING_TYPE",
+  74: "EXTERNAL_INTERNALIZED_STRING_WITH_ASCII_DATA_TYPE",
+  82: "SHORT_EXTERNAL_INTERNALIZED_STRING_TYPE",
+  86: "SHORT_EXTERNAL_ASCII_INTERNALIZED_STRING_TYPE",
+  90: "SHORT_EXTERNAL_INTERNALIZED_STRING_WITH_ASCII_DATA_TYPE",
+  128: "SYMBOL_TYPE",
+  129: "MAP_TYPE",
+  130: "CODE_TYPE",
+  131: "ODDBALL_TYPE",
+  132: "JS_GLOBAL_PROPERTY_CELL_TYPE",
+  133: "HEAP_NUMBER_TYPE",
+  134: "FOREIGN_TYPE",
+  135: "BYTE_ARRAY_TYPE",
+  136: "FREE_SPACE_TYPE",
+  137: "EXTERNAL_BYTE_ARRAY_TYPE",
+  138: "EXTERNAL_UNSIGNED_BYTE_ARRAY_TYPE",
+  139: "EXTERNAL_SHORT_ARRAY_TYPE",
+  140: "EXTERNAL_UNSIGNED_SHORT_ARRAY_TYPE",
+  141: "EXTERNAL_INT_ARRAY_TYPE",
+  142: "EXTERNAL_UNSIGNED_INT_ARRAY_TYPE",
+  143: "EXTERNAL_FLOAT_ARRAY_TYPE",
+  145: "EXTERNAL_PIXEL_ARRAY_TYPE",
+  147: "FILLER_TYPE",
+  148: "DECLARED_ACCESSOR_DESCRIPTOR_TYPE",
+  149: "DECLARED_ACCESSOR_INFO_TYPE",
+  150: "EXECUTABLE_ACCESSOR_INFO_TYPE",
+  151: "ACCESSOR_PAIR_TYPE",
+  152: "ACCESS_CHECK_INFO_TYPE",
+  153: "INTERCEPTOR_INFO_TYPE",
+  154: "CALL_HANDLER_INFO_TYPE",
+  155: "FUNCTION_TEMPLATE_INFO_TYPE",
+  156: "OBJECT_TEMPLATE_INFO_TYPE",
+  157: "SIGNATURE_INFO_TYPE",
+  158: "TYPE_SWITCH_INFO_TYPE",
+  159: "ALLOCATION_SITE_INFO_TYPE",
+  160: "SCRIPT_TYPE",
+  161: "CODE_CACHE_TYPE",
+  162: "POLYMORPHIC_CODE_CACHE_TYPE",
+  163: "TYPE_FEEDBACK_INFO_TYPE",
+  164: "ALIASED_ARGUMENTS_ENTRY_TYPE",
+  167: "FIXED_ARRAY_TYPE",
+  146: "FIXED_DOUBLE_ARRAY_TYPE",
+  168: "SHARED_FUNCTION_INFO_TYPE",
+  169: "JS_MESSAGE_OBJECT_TYPE",
+  172: "JS_VALUE_TYPE",
+  173: "JS_DATE_TYPE",
+  174: "JS_OBJECT_TYPE",
+  175: "JS_CONTEXT_EXTENSION_OBJECT_TYPE",
+  176: "JS_MODULE_TYPE",
+  177: "JS_GLOBAL_OBJECT_TYPE",
+  178: "JS_BUILTINS_OBJECT_TYPE",
+  179: "JS_GLOBAL_PROXY_TYPE",
+  180: "JS_ARRAY_TYPE",
+  171: "JS_PROXY_TYPE",
+  183: "JS_WEAK_MAP_TYPE",
+  184: "JS_REGEXP_TYPE",
+  185: "JS_FUNCTION_TYPE",
+  170: "JS_FUNCTION_PROXY_TYPE",
+  165: "DEBUG_INFO_TYPE",
+  166: "BREAK_POINT_INFO_TYPE",
 }
 
 
@@ -931,86 +931,87 @@ INSTANCE_TYPES = {
 # }
 # printf("}\n");
 KNOWN_MAPS = {
-  0x08081: (134, "ByteArrayMap"),
-  0x080a9: (128, "MetaMap"),
-  0x080d1: (130, "OddballMap"),
-  0x080f9: (68, "AsciiSymbolMap"),
-  0x08121: (166, "FixedArrayMap"),
-  0x08149: (132, "HeapNumberMap"),
-  0x08171: (135, "FreeSpaceMap"),
-  0x08199: (146, "OnePointerFillerMap"),
-  0x081c1: (146, "TwoPointerFillerMap"),
-  0x081e9: (131, "GlobalPropertyCellMap"),
-  0x08211: (167, "SharedFunctionInfoMap"),
-  0x08239: (4, "AsciiStringMap"),
-  0x08261: (166, "NativeContextMap"),
-  0x08289: (129, "CodeMap"),
-  0x082b1: (166, "ScopeInfoMap"),
-  0x082d9: (166, "FixedCOWArrayMap"),
-  0x08301: (145, "FixedDoubleArrayMap"),
-  0x08329: (166, "HashTableMap"),
+  0x08081: (135, "ByteArrayMap"),
+  0x080a9: (129, "MetaMap"),
+  0x080d1: (131, "OddballMap"),
+  0x080f9: (68, "AsciiInternalizedStringMap"),
+  0x08121: (167, "FixedArrayMap"),
+  0x08149: (133, "HeapNumberMap"),
+  0x08171: (136, "FreeSpaceMap"),
+  0x08199: (147, "OnePointerFillerMap"),
+  0x081c1: (147, "TwoPointerFillerMap"),
+  0x081e9: (132, "GlobalPropertyCellMap"),
+  0x08211: (168, "SharedFunctionInfoMap"),
+  0x08239: (167, "NativeContextMap"),
+  0x08261: (130, "CodeMap"),
+  0x08289: (167, "ScopeInfoMap"),
+  0x082b1: (167, "FixedCOWArrayMap"),
+  0x082d9: (146, "FixedDoubleArrayMap"),
+  0x08301: (167, "HashTableMap"),
+  0x08329: (128, "SymbolMap"),
   0x08351: (0, "StringMap"),
-  0x08379: (64, "SymbolMap"),
+  0x08379: (4, "AsciiStringMap"),
   0x083a1: (1, "ConsStringMap"),
   0x083c9: (5, "ConsAsciiStringMap"),
   0x083f1: (3, "SlicedStringMap"),
   0x08419: (7, "SlicedAsciiStringMap"),
-  0x08441: (65, "ConsSymbolMap"),
-  0x08469: (69, "ConsAsciiSymbolMap"),
-  0x08491: (66, "ExternalSymbolMap"),
-  0x084b9: (74, "ExternalSymbolWithAsciiDataMap"),
-  0x084e1: (70, "ExternalAsciiSymbolMap"),
-  0x08509: (2, "ExternalStringMap"),
-  0x08531: (10, "ExternalStringWithAsciiDataMap"),
-  0x08559: (6, "ExternalAsciiStringMap"),
-  0x08581: (82, "ShortExternalSymbolMap"),
-  0x085a9: (90, "ShortExternalSymbolWithAsciiDataMap"),
-  0x085d1: (86, "ShortExternalAsciiSymbolMap"),
-  0x085f9: (18, "ShortExternalStringMap"),
-  0x08621: (26, "ShortExternalStringWithAsciiDataMap"),
-  0x08649: (22, "ShortExternalAsciiStringMap"),
-  0x08671: (0, "UndetectableStringMap"),
-  0x08699: (4, "UndetectableAsciiStringMap"),
-  0x086c1: (144, "ExternalPixelArrayMap"),
-  0x086e9: (136, "ExternalByteArrayMap"),
-  0x08711: (137, "ExternalUnsignedByteArrayMap"),
-  0x08739: (138, "ExternalShortArrayMap"),
-  0x08761: (139, "ExternalUnsignedShortArrayMap"),
-  0x08789: (140, "ExternalIntArrayMap"),
-  0x087b1: (141, "ExternalUnsignedIntArrayMap"),
-  0x087d9: (142, "ExternalFloatArrayMap"),
-  0x08801: (143, "ExternalDoubleArrayMap"),
-  0x08829: (166, "NonStrictArgumentsElementsMap"),
-  0x08851: (166, "FunctionContextMap"),
-  0x08879: (166, "CatchContextMap"),
-  0x088a1: (166, "WithContextMap"),
-  0x088c9: (166, "BlockContextMap"),
-  0x088f1: (166, "ModuleContextMap"),
-  0x08919: (166, "GlobalContextMap"),
-  0x08941: (168, "JSMessageObjectMap"),
-  0x08969: (133, "ForeignMap"),
-  0x08991: (173, "NeanderMap"),
-  0x089b9: (158, "AllocationSiteInfoMap"),
-  0x089e1: (161, "PolymorphicCodeCacheMap"),
-  0x08a09: (159, "ScriptMap"),
-  0x08a31: (173, ""),
-  0x08a59: (173, "ExternalMap"),
-  0x08a81: (147, "DeclaredAccessorDescriptorMap"),
-  0x08aa9: (148, "DeclaredAccessorInfoMap"),
-  0x08ad1: (149, "ExecutableAccessorInfoMap"),
-  0x08af9: (150, "AccessorPairMap"),
-  0x08b21: (151, "AccessCheckInfoMap"),
-  0x08b49: (152, "InterceptorInfoMap"),
-  0x08b71: (153, "CallHandlerInfoMap"),
-  0x08b99: (154, "FunctionTemplateInfoMap"),
-  0x08bc1: (155, "ObjectTemplateInfoMap"),
-  0x08be9: (156, "SignatureInfoMap"),
-  0x08c11: (157, "TypeSwitchInfoMap"),
-  0x08c39: (160, "CodeCacheMap"),
-  0x08c61: (162, "TypeFeedbackInfoMap"),
-  0x08c89: (163, "AliasedArgumentsEntryMap"),
-  0x08cb1: (164, "DebugInfoMap"),
-  0x08cd9: (165, "BreakPointInfoMap"),
+  0x08441: (2, "ExternalStringMap"),
+  0x08469: (10, "ExternalStringWithAsciiDataMap"),
+  0x08491: (6, "ExternalAsciiStringMap"),
+  0x084b9: (18, "ShortExternalStringMap"),
+  0x084e1: (26, "ShortExternalStringWithAsciiDataMap"),
+  0x08509: (64, "InternalizedStringMap"),
+  0x08531: (65, "ConsInternalizedStringMap"),
+  0x08559: (69, "ConsAsciiInternalizedStringMap"),
+  0x08581: (66, "ExternalInternalizedStringMap"),
+  0x085a9: (74, "ExternalInternalizedStringWithAsciiDataMap"),
+  0x085d1: (70, "ExternalAsciiInternalizedStringMap"),
+  0x085f9: (82, "ShortExternalInternalizedStringMap"),
+  0x08621: (90, "ShortExternalInternalizedStringWithAsciiDataMap"),
+  0x08649: (86, "ShortExternalAsciiInternalizedStringMap"),
+  0x08671: (22, "ShortExternalAsciiStringMap"),
+  0x08699: (0, "UndetectableStringMap"),
+  0x086c1: (4, "UndetectableAsciiStringMap"),
+  0x086e9: (145, "ExternalPixelArrayMap"),
+  0x08711: (137, "ExternalByteArrayMap"),
+  0x08739: (138, "ExternalUnsignedByteArrayMap"),
+  0x08761: (139, "ExternalShortArrayMap"),
+  0x08789: (140, "ExternalUnsignedShortArrayMap"),
+  0x087b1: (141, "ExternalIntArrayMap"),
+  0x087d9: (142, "ExternalUnsignedIntArrayMap"),
+  0x08801: (143, "ExternalFloatArrayMap"),
+  0x08829: (144, "ExternalDoubleArrayMap"),
+  0x08851: (167, "NonStrictArgumentsElementsMap"),
+  0x08879: (167, "FunctionContextMap"),
+  0x088a1: (167, "CatchContextMap"),
+  0x088c9: (167, "WithContextMap"),
+  0x088f1: (167, "BlockContextMap"),
+  0x08919: (167, "ModuleContextMap"),
+  0x08941: (167, "GlobalContextMap"),
+  0x08969: (169, "JSMessageObjectMap"),
+  0x08991: (134, "ForeignMap"),
+  0x089b9: (174, "NeanderMap"),
+  0x089e1: (159, "AllocationSiteInfoMap"),
+  0x08a09: (162, "PolymorphicCodeCacheMap"),
+  0x08a31: (160, "ScriptMap"),
+  0x08a59: (174, ""),
+  0x08a81: (174, "ExternalMap"),
+  0x08aa9: (148, "DeclaredAccessorDescriptorMap"),
+  0x08ad1: (149, "DeclaredAccessorInfoMap"),
+  0x08af9: (150, "ExecutableAccessorInfoMap"),
+  0x08b21: (151, "AccessorPairMap"),
+  0x08b49: (152, "AccessCheckInfoMap"),
+  0x08b71: (153, "InterceptorInfoMap"),
+  0x08b99: (154, "CallHandlerInfoMap"),
+  0x08bc1: (155, "FunctionTemplateInfoMap"),
+  0x08be9: (156, "ObjectTemplateInfoMap"),
+  0x08c11: (157, "SignatureInfoMap"),
+  0x08c39: (158, "TypeSwitchInfoMap"),
+  0x08c61: (161, "CodeCacheMap"),
+  0x08c89: (163, "TypeFeedbackInfoMap"),
+  0x08cb1: (164, "AliasedArgumentsEntryMap"),
+  0x08cd9: (165, "DebugInfoMap"),
+  0x08d01: (166, "BreakPointInfoMap"),
 }
 
 
@@ -1145,8 +1146,104 @@ class HeapObject(object):
 
 
 class Map(HeapObject):
+  def Decode(self, offset, size, value):
+    return (value >> offset) & ((1 << size) - 1)
+
+  # Instance Sizes
+  def InstanceSizesOffset(self):
+    return self.heap.PointerSize()
+
+  def InstanceSizeOffset(self):
+    return self.InstanceSizesOffset()
+
+  def InObjectProperties(self):
+    return self.InstanceSizeOffset() + 1
+
+  def PreAllocatedPropertyFields(self):
+    return self.InObjectProperties() + 1
+
+  def VisitorId(self):
+    return self.PreAllocatedPropertyFields() + 1
+
+  # Instance Attributes
+  def InstanceAttributesOffset(self):
+    return self.InstanceSizesOffset() + self.heap.IntSize()
+
   def InstanceTypeOffset(self):
-    return self.heap.PointerSize() + self.heap.IntSize()
+    return self.InstanceAttributesOffset()
+
+  def UnusedPropertyFieldsOffset(self):
+    return self.InstanceTypeOffset() + 1
+
+  def BitFieldOffset(self):
+    return self.UnusedPropertyFieldsOffset() + 1
+
+  def BitField2Offset(self):
+    return self.BitFieldOffset() + 1
+
+  # Other fields
+  def PrototypeOffset(self):
+    return self.InstanceAttributesOffset() + self.heap.IntSize()
+
+  def ConstructorOffset(self):
+    return self.PrototypeOffset() + self.heap.PointerSize()
+
+  def TransitionsOrBackPointerOffset(self):
+    return self.ConstructorOffset() + self.heap.PointerSize()
+
+  def DescriptorsOffset(self):
+    return self.TransitionsOrBackPointerOffset() + self.heap.PointerSize()
+
+  def CodeCacheOffset(self):
+    return self.DescriptorsOffset() + self.heap.PointerSize()
+
+  def DependentCodeOffset(self):
+    return self.CodeCacheOffset() + self.heap.PointerSize()
+
+  def BitField3Offset(self):
+    return self.DependentCodeOffset() + self.heap.PointerSize()
+
+  def ReadByte(self, offset):
+    return self.heap.reader.ReadU8(self.address + offset)
+
+  def Print(self, p):
+    p.Print("Map(%08x)" % (self.address))
+    p.Print("- size: %d, inobject: %d, preallocated: %d, visitor: %d" % (
+        self.ReadByte(self.InstanceSizeOffset()),
+        self.ReadByte(self.InObjectProperties()),
+        self.ReadByte(self.PreAllocatedPropertyFields()),
+        self.VisitorId()))
+
+    bitfield = self.ReadByte(self.BitFieldOffset())
+    bitfield2 = self.ReadByte(self.BitField2Offset())
+    p.Print("- %s, unused: %d, bf: %d, bf2: %d" % (
+        INSTANCE_TYPES[self.ReadByte(self.InstanceTypeOffset())],
+        self.ReadByte(self.UnusedPropertyFieldsOffset()),
+        bitfield, bitfield2))
+
+    p.Print("- kind: %s" % (self.Decode(3, 5, bitfield2)))
+
+    bitfield3 = self.ObjectField(self.BitField3Offset())
+    p.Print(
+        "- EnumLength: %d NumberOfOwnDescriptors: %d OwnsDescriptors: %s" % (
+            self.Decode(0, 11, bitfield3),
+            self.Decode(11, 11, bitfield3),
+            self.Decode(25, 1, bitfield3)))
+    p.Print("- IsShared: %s" % (self.Decode(22, 1, bitfield3)))
+    p.Print("- FunctionWithPrototype: %s" % (self.Decode(23, 1, bitfield3)))
+    p.Print("- DictionaryMap: %s" % (self.Decode(24, 1, bitfield3)))
+
+    descriptors = self.ObjectField(self.DescriptorsOffset())
+    if descriptors.__class__ == FixedArray:
+      DescriptorArray(descriptors).Print(p)
+    else:
+      p.Print("Descriptors: %s" % (descriptors))
+
+    transitions = self.ObjectField(self.TransitionsOrBackPointerOffset())
+    if transitions.__class__ == FixedArray:
+      TransitionArray(transitions).Print(p)
+    else:
+      p.Print("TransitionsOrBackPointer: %s" % (transitions))
 
   def __init__(self, heap, map, address):
     HeapObject.__init__(self, heap, map, address)
@@ -1280,6 +1377,12 @@ class FixedArray(HeapObject):
   def ElementsOffset(self):
     return self.heap.PointerSize() * 2
 
+  def MemberOffset(self, i):
+    return self.ElementsOffset() + self.heap.PointerSize() * i
+
+  def Get(self, i):
+    return self.ObjectField(self.MemberOffset(i))
+
   def __init__(self, heap, map, address):
     HeapObject.__init__(self, heap, map, address)
     self.length = self.SmiField(self.LengthOffset())
@@ -1305,6 +1408,111 @@ class FixedArray(HeapObject):
     return "FixedArray(%08x, length=%d)" % (self.address, self.length)
 
 
+class DescriptorArray(object):
+  def __init__(self, array):
+    self.array = array
+
+  def Length(self):
+    return self.array.Get(0)
+
+  def Decode(self, offset, size, value):
+    return (value >> offset) & ((1 << size) - 1)
+
+  TYPES = [
+      "normal",
+      "field",
+      "function",
+      "callbacks"
+  ]
+
+  def Type(self, value):
+    return DescriptorArray.TYPES[self.Decode(0, 3, value)]
+
+  def Attributes(self, value):
+    attributes = self.Decode(3, 3, value)
+    result = []
+    if (attributes & 0): result += ["ReadOnly"]
+    if (attributes & 1): result += ["DontEnum"]
+    if (attributes & 2): result += ["DontDelete"]
+    return "[" + (",".join(result)) + "]"
+
+  def Deleted(self, value):
+    return self.Decode(6, 1, value) == 1
+
+  def Storage(self, value):
+    return self.Decode(7, 11, value)
+
+  def Pointer(self, value):
+    return self.Decode(18, 11, value)
+
+  def Details(self, di, value):
+    return (
+        di,
+        self.Type(value),
+        self.Attributes(value),
+        self.Storage(value),
+        self.Pointer(value)
+    )
+
+
+  def Print(self, p):
+    length = self.Length()
+    array = self.array
+
+    p.Print("Descriptors(%08x, length=%d)" % (array.address, length))
+    p.Print("[et] %s" % (array.Get(1)))
+
+    for di in xrange(length):
+      i = 2 + di * 3
+      p.Print("0x%x" % (array.address + array.MemberOffset(i)))
+      p.Print("[%i] name:    %s" % (di, array.Get(i + 0)))
+      p.Print("[%i] details: %s %s enum %i pointer %i" % \
+              self.Details(di, array.Get(i + 1)))
+      p.Print("[%i] value:   %s" % (di, array.Get(i + 2)))
+
+    end = self.array.length // 3
+    if length != end:
+      p.Print("[%i-%i] slack descriptors" % (length, end))
+
+
+class TransitionArray(object):
+  def __init__(self, array):
+    self.array = array
+
+  def IsSimpleTransition(self):
+    return self.array.length <= 2
+
+  def Length(self):
+    # SimpleTransition cases
+    if self.IsSimpleTransition():
+      return self.array.length - 1
+    return (self.array.length - 3) // 2
+
+  def Print(self, p):
+    length = self.Length()
+    array = self.array
+
+    p.Print("Transitions(%08x, length=%d)" % (array.address, length))
+    p.Print("[backpointer] %s" % (array.Get(0)))
+    if self.IsSimpleTransition():
+      if length == 1:
+        p.Print("[simple target] %s" % (array.Get(1)))
+      return
+
+    elements = array.Get(1)
+    if elements is not None:
+      p.Print("[elements   ] %s" % (elements))
+
+    prototype = array.Get(2)
+    if prototype is not None:
+      p.Print("[prototype  ] %s" % (prototype))
+
+    for di in xrange(length):
+      i = 3 + di * 2
+      p.Print("[%i] symbol: %s" % (di, array.Get(i + 0)))
+      p.Print("[%i] target: %s" % (di, array.Get(i + 1)))
+
+
 class JSFunction(HeapObject):
   def CodeEntryOffset(self):
     return 3 * self.heap.PointerSize()
@@ -1720,6 +1928,30 @@ class InspectionShell(cmd.Cmd):
     else:
       print "Address cannot be interpreted as object!"
 
+  def do_do_desc(self, address):
+    """
+      Print a descriptor array in a readable format.
+    """
+    start = int(address, 16)
+    if ((start & 1) == 1): start = start - 1
+    DescriptorArray(FixedArray(self.heap, None, start)).Print(Printer())
+
+  def do_do_map(self, address):
+    """
+      Print a descriptor array in a readable format.
+    """
+    start = int(address, 16)
+    if ((start & 1) == 1): start = start - 1
+    Map(self.heap, None, start).Print(Printer())
+
+  def do_do_trans(self, address):
+    """
+      Print a transition array in a readable format.
+    """
+    start = int(address, 16)
+    if ((start & 1) == 1): start = start - 1
+    TransitionArray(FixedArray(self.heap, None, start)).Print(Printer())
+
   def do_dp(self, address):
     """
      Interpret memory at the given address as being on a V8 heap page
@@ -1854,13 +2086,14 @@ def AnalyzeMinidump(options, minidump_name):
     else:
       print "    eflags: %s" % bin(reader.exception_context.eflags)[2:]
 
-    print
-    print "  modules:"
-    for module in reader.module_list.modules:
-      name = GetModuleName(reader, module)
-      if name in KNOWN_MODULES:
-        print "    %s at %08X" % (name, module.base_of_image)
-        reader.TryLoadSymbolsFor(name, module)
+    # TODO(mstarzinger): Disabled because broken, needs investigation.
+    #print
+    #print "  modules:"
+    #for module in reader.module_list.modules:
+    #  name = GetModuleName(reader, module)
+    #  if name in KNOWN_MODULES:
+    #    print "    %s at %08X" % (name, module.base_of_image)
+    #    reader.TryLoadSymbolsFor(name, module)
     print
 
     stack_top = reader.ExceptionSP()
